// calculate total pixel number and image data
// usage : img_tot file1 x1 x2 y1 y2


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*----------------------------------------------------------------------*/
#define M_PI				3.1415926535897932385
#define INTEL
#define HiPic_Header_Size	64
#define MAXSHOT				5000
#define MAXPIXL				2048

struct HiPic_Header
{
	char					head[2];
	unsigned short			comment_length;
	unsigned short			width;
	unsigned short			height;
	unsigned short			x_offset;
	unsigned short			y_offset;
	unsigned short			type;
	char					reserved[63 - 13];
	char					*comment;
};
typedef struct HiPic_Header Header;

/*----------------------------------------------------------------------*/

int					x1, y1;
unsigned short		data[MAXPIXL][MAXPIXL]; // data[y][x]

/*----------------------------------------------------------------------*/
int read_hipic(char *fname, unsigned short *data, Header *h)
	{
	int			x_size, y_size;
	int			i,j;
	FILE		*fi;
	short		d_dum;
	char		s_dum, s_time[10];
	float		f_dum;
	char		s1[38], s2[10];

//open input files
		if((fi = fopen(fname,"rb")) == NULL)
			{ printf("can not open %s for input\n", fname); return(-10);}
// read comment and image data from file1
		if (fread(h, sizeof(char), HiPic_Header_Size, fi) != HiPic_Header_Size){
			fprintf(stderr, "EOF in %s (header)\n", fname);
			return(-1);
		}
		if (strncmp(h->head, "IM", 2)){
			fclose(fi);
			fprintf(stderr, "This is not HiPIc image file.\n");
			return(-11);
		}
		
		h->comment = (char *) malloc(h->comment_length + 1);
		if (h->comment == NULL){
			fclose(fi);
			fprintf(stderr, "Connot allocate memory (HiPic_Header.comment)\n");
			return(-12);
		}
		if (fread(h->comment, sizeof(char), h->comment_length, fi) != h->comment_length){
			fclose(fi);
			free(h->comment);
			fprintf(stderr, "EOF found (comment)\n");
			return(-13);
		}
		x_size = h->width; // printf("x_size         %d \n", x_size);
		y_size = h->height;// printf("y_size         %d \n", y_size);
//		printf("comment length %d \n\n", h->comment_length);

//		sscanf(h->comment,"%s %f %d %f %d %d %d %d %d %d %d %s %s",
//							s_dum, &f_dum, &d_dum, &f_dum, &d_dum,
//							&d_dum, &d_dum, &d_dum, &d_dum, &d_dum, &d_dum,
//							s_dum, s_time);
		for(i=0; i<7; ++i){
			if((&h->comment[38+i])!="m"){
				s1[i]=h->comment[38+i];
			}
		}

//			printf("%s, ", s1);

//printf("%c\n", s_dum);
//printf("%d\n", d_dum);
//printf("%f\n", f_dum);
//printf("%s\n", s_time);

//exit(0);
		h->comment[h->comment_length] = '\0';

		for(i = 0; i < h->height; ++i){
			if ((j = fread(data + MAXPIXL*i, sizeof(unsigned short), h->width, fi)) != h->width){
				fprintf(stderr, " Error reading %s at %d (%d)\n", fname, i, j);
				fclose(fi);
				free(h->comment);
				return(i+1);
			}
		}
// check for correct reading of HiPic file
//printf("-------------------------------\n");
//printf("%s\n", fname);
//printf("%s\n", h->head);
//printf("%d\n", h->comment_length);
//printf("%d\n", h->width);
//printf("%d\n", h->height);
//printf("%d\n", h->x_offset);
//printf("%d\n", h->y_offset);
//printf("%d\n", h->type);
//printf("%s\n", h->reserved);
//printf("%s\n", h->comment);
//printf("-------------------------------\n");

		fclose(fi);
		free(h->comment);
		return(0);
	}

/*----------------------------------------------------------------------*/

int main(argc,argv)
int		argc;
char	**argv;
{
	Header		h;
	
	int			i, j, imax, imin, jmax, jmin;
	unsigned short		dmax, dmin, x1, x2, y1, y2;
	unsigned long		dtotal;

	if (argc<2){
		printf("  usage : img_tot file1 x1 x2 y1 y2\n");
		return (-1);
	}

	dmax   = -999;
	dmin   =  16300;
	dtotal =    0;

// read file
	if ((i = read_hipic(argv[1], &data[0][0], &h)) != 0){
		printf("something wrong -- return value is %d", i);
		exit(-1);
	}
	x1 = 0;
	x2 = h.width;
	y1 = 0;
	y2 = h.height;

	if (argc==6){
		x1 = atoi(argv[2]);
		x2 = atoi(argv[3]);
		y1 = atoi(argv[4]);
		y2 = atoi(argv[5]);
	}

//printf("image is  ---   h.height = %d, h.width = %d\n", h.height, h.width);
//	printf("\n");
//	for(i = 0; i < h.height; i++)
	for(i = y1; i < y2; i++){
//		for (j = x1; j < h.width; j++)
		for (j = x1; j < x2; j++){
			if (dmax < data[i][j]){
				dmax = data[i][j];
				imax = i;
				jmax = j;
			}
			if (dmin > data[i][j]){
				dmin = data[i][j];
				imin = i;
				jmin = j;
			}
			dtotal = dtotal + data[i][j];
		}
	}
//	printf("file = %s \n", argv[1]);
//	printf("Image width   = %10d           \n", h.width);
//	printf("      height  = %10d           \n", h.height);
//	printf("Image width   = %10d           \n", x2-x1);
//	printf("      height  = %10d           \n", y2-y1);
//	printf("Maximum value = %10d (%4d,%4d) \n", dmax, jmax, imax);
//	printf("Minimum value = %10d (%4d,%4d) \n", dmin, jmin, imin);
//	printf("Total   value = %10d           \n", dtotal);
//	printf("Average value = %10d           \n", dtotal/((x2-x1)*(y2-y1)));
//	printf("%d, %d, %d\n", x2-x1, y2-y1, dtotal/((x2-x1)*(y2-y1)));
	printf("%d, %d, %d\n", x2-x1, y2-y1, dtotal);

	return 0;
}
